
# THIS FILE IS GENERATED FROM NUMPY SETUP.PY
short_version = '1.7.2'
version = '1.7.2'
full_version = '1.7.2'
git_revision = '4322701029c8bf1d87203cafe781a0128dd86a62'
release = True

if not release:
    version = full_version
